library(plotrix)  #package to draw circles easily
library(doParallel)  # Package to parallelize computations
library(TDA)  #package to compute persistent (with Delaunay-complex filtrations)
library(R.matlab)  # To import .mat file


# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat and to return a
# matrix in place of object of class diagram
alphaDiag <- function(PP) {
    if (is.ppp(PP) == TRUE) {
        sim_PP_coord <- cbind(PP$x, PP$y)
    } else {
        sim_PP_coord <- cbind(PP[, 1], PP[, 2])
    }
    Diag <- cbind(alphaComplexDiag(sim_PP_coord)$diagram[, 1], sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 2]), 
                  sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 3]))
    colnames(Diag) <- c("dimension", "Birth", "Death")
    return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).  m: the arguments of the APF, Diag: the persistence
# diagram, k: the dimension of the topological features
APFtemp <- function(m, Diag, k) {
    Diagk <- Diag[Diag[, 1] == k, ]
    if (is.matrix(Diagk) == TRUE) {
        lifetime <- Diagk[, 3] - Diagk[, 2]
        meanage <- 0.5 * (Diagk[, 3] + Diagk[, 2])
        sum(lifetime[meanage <= m])
    } else {
        lifetime <- Diagk[3] - Diagk[2]
        meanage <- 0.5 * (Diagk[3] + Diagk[2])
        sum(lifetime[meanage <= m])
    }
}
APF <- Vectorize(APFtemp, vectorize.args = "m", "Diag")

# Loading of the persistence diagrams
CaseInformation <- readMat("CaseInformation.mat")
H0.PD.temp <- readMat("H0PersistenceDiagrams.mat")
H1.PD.temp <- readMat("H1PersistenceDiagrams.mat")

# Conversion into a list of matrices
H0.PD.full <- NULL
H1.PD.full <- NULL
for (i in (1:length(H0.PD.temp$H0Array))) {
    H0.PD.full[[i]] <- cbind(rep(0, length(H0.PD.temp$H0Array[[i]][[1]][, 1])), H0.PD.temp$H0Array[[i]][[1]])
}
for (i in (1:length(H1.PD.temp$H1Array))) {
    H1.PD.full[[i]] <- cbind(rep(1, length(H1.PD.temp$H1Array[[i]][[1]][, 1])), H1.PD.temp$H1Array[[i]][[1]])
}

PD.brain.full <- NULL
for (i in 1:length(H0.PD.full)) {
    PD.brain.full[[i]] <- rbind(rbind(H0.PD.full[[i]], H1.PD.full[[i]]))
}

# #### Empty subject detection, I take off them from the sample
Emptysubject0 <- NULL
Emptysubject1 <- NULL
for (i in 1:109) {
    Emptysubject0[i] <- length(H0.PD.full[[i]])
}
Emptysubject0 <- which(Emptysubject0 == 0)
Emptysubject1 <- which(Emptysubject1 == 0)
H0.PD <- H0.PD.full[-Emptysubject0]
H1.PD <- H1.PD.full[-Emptysubject0]

# I replace the value -1 in the persistence diagram by Inf, these values correspond to unobserved death time.
for (i in (1:length(H1.PD))) {
    H1.PD[[i]][which(H1.PD[[i]][, 3] == -1), 3] <- Inf
}

#### I compute all the persistence diagrams that are non empty
PD.brain <- NULL
for (i in 1:length(H0.PD)) {
    PD.brain[[i]] <- rbind(rbind(H0.PD[[i]], H1.PD[[i]]))
}

# Index of the female and male subjects
female <- which(CaseInformation$Sexes[-Emptysubject0] == 1)
male <- which(CaseInformation$Sexes[-Emptysubject0] == 2)

# Computation of APF0 corresponding to female and male subject
mm0 <- seq(0, 160, 0.005)
registerDoParallel(cores = 3)
APF0.brain.female <- foreach(i = female, .combine = rbind) %dopar% {
    APF(mm0, PD.brain[[i]], 0)
}
APF0.brain.male <- foreach(i = male, .combine = rbind) %dopar% {
    APF(mm0, PD.brain[[i]], 0)
}

# Computation of APF0 corresponding to female and male subject
mm1 <- seq(0, 27, 1e-04)
registerDoParallel(cores = 3)
APF1.brain.female <- foreach(i = female, .combine = rbind) %dopar% {
    APF(mm1, PD.brain[[i]], 1)
}
APF1.brain.male <- foreach(i = male, .combine = rbind) %dopar% {
    APF(mm1, PD.brain[[i]], 1)
}




# Function to compute the size of the confidence region at level 95% AAPF: the vector of the values of the APF,
# mcores: nbre of cores used, BB: number of iteration of the bootstrap method.
width.conf.band.brain <- function(AAPF, mcores, BB) {
    registerDoParallel(cores = mcores)
    N <- length(AAPF[, 1])
    quant0 <- foreach(i = 1:BB, .combine = rbind) %dopar% {
        Nboot <- sample(N, N, replace = T)
        Pboot_APF <- AAPF[Nboot, ]
        max(sqrt(N) * abs(colMeans(Pboot_APF) - colMeans(AAPF)))
    }
    quantile(quant0, probs = 0.95, names = F)/sqrt(N)
}

# Computation of the quantiles for each confidence region.
q0.female <- width.conf.band.brain(APF0.brain.female, mcores = 2, BB = 10000)
q0.male <- width.conf.band.brain(APF0.brain.male, mcores = 2, BB = 10000)
q1.female <- width.conf.band.brain(APF1.brain.female, mcores = 2, BB = 10000)
q1.male <- width.conf.band.brain(APF1.brain.male, mcores = 2, BB = 10000)



# Code for figure 7 Left panel
par(mar = c(7, 7, 0.5, 0.5))
plot(mm0[seq(1, length(mm0), by = 10)], colMeans(APF0.brain.male[, seq(1, length(mm0), by = 10)]), type = "l", xlim = c(0, 
    130), ylim = c(0, 1420), main = "", xlab = "", ylab = "", col = 0, cex.axis = 2.5, mgp = c(3, 2, 0))
# Bound for Females
lines(mm0[seq(1, length(mm0), by = 10)], colMeans(APF0.brain.female[, seq(1, length(mm0), by = 10)]) + q0.female, col = 4, 
    lwd = 4)
lines(mm0[seq(1, length(mm0), by = 10)], pmax(0, colMeans(APF0.brain.female[, seq(1, length(mm0), by = 10)]) - q0.female), 
    col = 4, lwd = 4)
# Bound for Males
lines(mm0[seq(1, length(mm0), by = 10)], colMeans(APF0.brain.male[, seq(1, length(mm0), by = 10)]) + q0.male, col = 2, 
    lwd = 4, lty = 2)
lines(mm0[seq(1, length(mm0), by = 10)], pmax(0, colMeans(APF0.brain.male[, seq(1, length(mm0), by = 10)]) - q0.male), 
    col = 2, lwd = 4, lty = 2)
title(ylab = expression(APF[0](m)), mgp = c(4, 1, 0), cex.lab = 3, lwd = 6)
title(xlab = expression("m"), mgp = c(5, 1, 0), cex.lab = 3)
legend("bottomright", c(expression(atop("Confidence region", " bounds - Female")), expression(atop("Confidence region", 
    " bounds - Male"))), col = c("4", "2"), lwd = c(6, 6), lty = c(1, 2), y.intersp = 2, cex = 2, seg.len = 3)

# Code for Figure 7 Right panel
par(mar = c(7, 7, 0.5, 0.5))
plot(mm1[seq(1, length(mm1), by = 100)], colMeans(APF1.brain.male[, seq(1, length(mm1), by = 100)]), type = "l", xlim = c(0, 
    25), ylim = c(0, 1020), main = "", xlab = "", ylab = "", col = 0, cex.axis = 2.5, mgp = c(3, 2, 0))
# Bound for Females
lines(mm1[seq(1, length(mm1), by = 100)], colMeans(APF1.brain.female[, seq(1, length(mm1), by = 100)]) + q1.female, col = 4, 
    lwd = 4)
lines(mm1[seq(1, length(mm1), by = 100)], pmax(0, colMeans(APF1.brain.female[, seq(1, length(mm1), by = 100)]) - q1.female), 
    col = 4, lwd = 4)
# Bound for Males
lines(mm1[seq(1, length(mm1), by = 100)], colMeans(APF1.brain.male[, seq(1, length(mm1), by = 100)]) + q1.male, col = 2, 
    lwd = 4, lty = 2)
lines(mm1[seq(1, length(mm1), by = 100)], pmax(0, colMeans(APF1.brain.male[, seq(1, length(mm1), by = 100)]) - q1.male), 
    col = 2, lwd = 4, lty = 2)
title(ylab = expression(APF[1](m)), mgp = c(4, 1, 0), cex.lab = 3, lwd = 6)
title(xlab = expression("m"), mgp = c(5, 1, 0), cex.lab = 3)
legend("bottomright", c(expression(atop("Confidence region", " bounds - Female")), expression(atop("Confidence region", 
    " bounds - Male"))), col = c("4", "2"), lwd = c(6, 6), lty = c(1, 2), y.intersp = 2, cex = 2, seg.len = 3)



