library(spatstat)
library(plotrix)  #package to draw circles easily
library(doParallel)  # Package to parallelize computations
library(TDA)  #package to compute  Delaunay-complex filtrations

# Modification of the TDA function to be compatible with ppp objects from the R-package spatstat and to return a
# matrix in place of object of class diagram
alphaDiag <- function(PP) {
    if (is.ppp(PP) == TRUE) {
        sim_PP_coord <- cbind(PP$x, PP$y)
    } else {
        sim_PP_coord <- cbind(PP[, 1], PP[, 2])
    }
    Diag <- cbind(alphaComplexDiag(sim_PP_coord)$diagram[, 1], sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 2]), 
                  sqrt(alphaComplexDiag(sim_PP_coord)$diagram[, 3]))
    colnames(Diag) <- c("dimension", "Birth", "Death")
    return(Diag)
}

# Definition of the Accumulated Persistent Function (APF).  m: the arguments of the APF, Diag: the persistence
# diagram, k: the dimension of the topological features
APFtemp <- function(m, Diag, k) {
    Diagk <- Diag[Diag[, 1] == k, ]
    if (is.matrix(Diagk) == TRUE) {
        lifetime <- Diagk[, 3] - Diagk[, 2]
        meanage <- 0.5 * (Diagk[, 3] + Diagk[, 2])
        sum(lifetime[meanage <= m])
    } else {
        lifetime <- Diagk[3] - Diagk[2]
        meanage <- 0.5 * (Diagk[3] + Diagk[2])
        sum(lifetime[meanage <= m])
    }
}
APF <- Vectorize(APFtemp, vectorize.args = "m", "Diag")

# Function performing the two-sample test based on the function KS defined as in (6). Return 1 if H0 is rejected at
# 95%.
# mm: arguments of the APF, Diag: list containing the repetition of D0, Qiag: list containing the repetition of
# E0, kk: dimension of the topological features considered, B: number of iterations of the bootstrap method
KS.permtest <- function(mm, Diag, Qiag, kk, B) {
    LL1 <- length(Diag)
    LL2 <- length(Qiag)
    Diag_APF <- foreach(ii = 1:LL1, .combine = rbind) %do% {
        APF(mm, Diag = Diag[[ii]], k = kk)
    }
    Qiag_APF <- foreach(ii = 1:LL2, .combine = rbind) %do% {
        APF(mm, Diag = Qiag[[ii]], k = kk)
    }
    Mat_APF <- rbind(Diag_APF, Qiag_APF)
    KS0 <- sqrt(LL1 * LL2/(LL1 + LL2)) * max(abs(colMeans(Mat_APF[1:LL1, ]) - colMeans(Mat_APF[(LL1 + 1):(LL1 + LL2), 
        ])))
    Bootapf <- function(MMat_APF) {
        bootMat_APF <- MMat_APF[sample(nrow(MMat_APF), nrow(MMat_APF), replace = FALSE, prob = NULL), ]
        sqrt(LL1 * LL2/(LL1 + LL2)) * max(abs(colMeans(bootMat_APF[1:LL1, ]) - colMeans(bootMat_APF[(LL1 + 1):(LL1 + 
            LL2), ])))
    }
    
    KS <- foreach(ii = 1:B, .combine = cbind) %do% {
        Bootapf(Mat_APF)
    }
    return(1 * (KS0 > quantile(KS, names = FALSE, 0.95)))
}


# Function to estimate the power of the test in the setting of example 8. nbrepoint: number of points of each point
# pattern, r1 and r2: radius of the circles, NN: size of each samples of persistence diagrams, kk: dimension of the
# topological features considered, BB: number of iterations of the bootstrap method, noiseamp= amplitude of the
# perturbation.
estim.puissance.twosample.KS <- function(nbrepoint, r1, r2, NN, kk, BB, noiseamp) {
    P1 <- NULL
    P2 <- NULL
    for (i in 1:NN) {
        P1[[i]] <- circleUnif(nbrepoint, r = r1) + cbind(rnorm(nbrepoint, 0, noiseamp), rnorm(nbrepoint, 0, noiseamp))
        P2[[i]] <- circleUnif(nbrepoint, r = r2) + cbind(rnorm(nbrepoint, 0, noiseamp), rnorm(nbrepoint, 0, noiseamp))
    }
    DiagP1 <- NULL
    DiagP2 <- NULL
    DiagP1 <- foreach(ii = 1:NN) %do% {
        alphaDiag(P1[[ii]])
    }
    DiagP2 <- foreach(ii = 1:NN) %do% {
        alphaDiag(P2[[ii]])
    }
    mm <- seq(0, 1, 0.001)
    KS.permtest(mm, DiagP1, DiagP2, kk = kk, B = BB)
}

# Code to run the test 500 hundred times
registerDoParallel(cores = 2)
# Using connected components
puissance0 <- foreach(ii = 1:500, .combine = c) %dopar% {
    estim.puissance.twosample.KS(nbrepoint = 100, r1 = 1, r2 = 0.95, NN = 50, kk = 0, BB = 1000, noiseamp = 0.2)
}
100 * mean(puissance0)  # display the estimated percentage of rejection
# Using loops
puissance1 <- foreach(ii = 1:500, .combine = c) %dopar% {
    estim.puissance.twosample.KS(nbrepoint = 100, r1 = 1, r2 = 0.95, NN = 50, kk = 1, BB = 1000, noiseamp = 0.2)
}
100 * mean(puissance1)  # display the estimated percentage of rejection



# Code for figure 13
par(mar = c(0, 0, 0, 0))
plot(circleUnif(100, r = 1) + cbind(rnorm(100, 0, 0.2), rnorm(100, 0, 0.2)), cex = 2.5, pch = 20, col = 2, asp = 1, 
     xlim = c(-1.9, 1.4), ylim = c(-1.4, 1.9), xlab = "", ylab = "", main = "", axes = F)
points(circleUnif(100, r = 0.95) + cbind(rnorm(100, 0, 0.2), rnorm(100, 0, 0.2)), cex = 2.1, col = 4, pch = 3, lwd = 4)
legend("topleft", c("Circle r=1", "Circle r=0.95"), col = c("2", "4"), pch = c(20, 3), lty = 0, y.intersp = 1.5, lwd = 4, 
      cex = 2.3, seg.len = 3, bty = "n")







# The code for the simulation when using the function M defined as in (10) in place of the function KS defined as in
# (6).


# Function performing the two-sample test based on the function KS defined as in (6). Return 1 if H0 is rejected at
# 95% mm: arguments of the APF, Diag: list containing the repetition of D0, Qiag: list containing the repetition of
# E0 B: number of iteration of the bootstrap method
M.permtest <- function(mm, Diag, Qiag, kk, B) {
    LL1 <- length(Diag)
    LL2 <- length(Qiag)
    Diag_APF <- foreach(ii = 1:LL1, .combine = rbind) %do% {
        APF(mm, Diag = Diag[[ii]], k = kk)
    }
    Qiag_APF <- foreach(ii = 1:LL2, .combine = rbind) %do% {
        APF(mm, Diag = Qiag[[ii]], k = kk)
    }
    Mat_APF <- rbind(Diag_APF, Qiag_APF)
    M0 <- sqrt(LL1 * LL2/(LL1 + LL2)) * mean(abs(colMeans(Mat_APF[1:LL1, ]) - colMeans(Mat_APF[(LL1 + 1):(LL1 + LL2), 
        ])))
    BootMapf <- function(MMat_APF) {
        bootMat_APF <- MMat_APF[sample(nrow(MMat_APF), nrow(MMat_APF), replace = FALSE, prob = NULL), ]
        sqrt(LL1 * LL2/(LL1 + LL2)) * mean(abs(colMeans(bootMat_APF[1:LL1, ]) - colMeans(bootMat_APF[(LL1 + 1):(LL1 + 
            LL2), ])))
    }
    
    Mboot <- foreach(ii = 1:B, .combine = cbind) %do% {
        BootMapf(Mat_APF)
    }
    return(1 * (M0 > quantile(Mboot, names = FALSE, 0.95)))
}

# Function to estimate the power of the test in the setting of example 8 nbrepoint: numver of points of each point
# pattern, r1 and r2: radius of the circles, NN: size of each samples of persistence diagrams, kk: dimension of the
# topological features considered, BB: number of iteration of the bootstrap method, noiseamp= amplitude of the
# perturbation.
estim.puissance.twosample.M <- function(nbrepoint, r1, r2, NN, kk, BB, noiseamp) {
    P1 <- NULL
    P2 <- NULL
    for (i in 1:NN) {
        P1[[i]] <- circleUnif(nbrepoint, r = r1) + cbind(rnorm(nbrepoint, 0, noiseamp), rnorm(nbrepoint, 0, noiseamp))
        P2[[i]] <- circleUnif(nbrepoint, r = r2) + cbind(rnorm(nbrepoint, 0, noiseamp), rnorm(nbrepoint, 0, noiseamp))
    }
    DiagP1 <- NULL
    DiagP2 <- NULL
    DiagP1 <- foreach(ii = 1:NN) %do% {
        alphaDiag(P1[[ii]])
    }
    DiagP2 <- foreach(ii = 1:NN) %do% {
        alphaDiag(P2[[ii]])
    }
    mm <- seq(0, 1, 0.001)
    M.permtest(mm, DiagP1, DiagP2, kk = kk, B = BB)
}


# Code to run the test 500 hundred times
registerDoParallel(cores = 3)
# Using connected components
puissance0.M <- foreach(ii = 1:500, .combine = c) %dopar% {
    estim.puissance.twosample.M(nbrepoint = 100, r1 = 1, r2 = 0.95, NN = 50, kk = 0, BB = 1000, noiseamp = 0.2)
}
100 * mean(puissance0.M)  # display the estimated percentage of rejection
# Using loops
puissance1.M <- foreach(ii = 1:500, .combine = c) %dopar% {
    estim.puissance.twosample.M(nbrepoint = 100, r1 = 1, r2 = 0.95, NN = 50, kk = 1, BB = 1000, noiseamp = 0.2)
}
100 * mean(puissance1.M)  # display the estimated percentage of rejection



